IF NOT OBJECT_ID('dbo._CVP_CHECKGRANT') IS NULL DROP PROCEDURE dbo._CVP_CHECKGRANT
GO
CREATE PROCEDURE dbo._CVP_CHECKGRANT
AS
BEGIN
 DECLARE @lorsData CURSOR
 DECLARE @lcNAME NVARCHAR(128), @liEXISTS INT, @lcLIST NVARCHAR(MAX)

 PRINT 'bulk admin role: '+ CASE WHEN IS_SRVROLEMEMBER('bulkadmin')=1 THEN 'Yes' ELSE 'No' END

 IF ISNULL(HAS_DBACCESS('master'),0)=0 BEGIN
    SET @lcLIST='/*****************************************/'+CHAR(10)+
                '/** RUN this                            **/'+CHAR(10)+
                '/*****************************************/'+CHAR(10)+
                REPLACE('USE master; CREATE USER [%LOGIN%] FOR LOGIN [%LOGIN%];'+CHAR(10)+'GO'+CHAR(10),'%LOGIN%',SYSTEM_USER)
    PRINT @lcLIST
    END
 ELSE BEGIN
    PRINT REPLACE('USER %LOGIN% has access to database master', '%LOGIN%',SYSTEM_USER)
 END

 IF (SELECT COUNT(1) FROM fn_my_permissions(NULL, 'server') WHERE permission_name='ALTER TRACE')=0 BEGIN
    SET @lcLIST='/*****************************************/'+CHAR(10)+
                '/** RUN this                            **/'+CHAR(10)+
                '/*****************************************/'+CHAR(10)+
                REPLACE('GRANT ALTER TRACE TO [%LOGIN%];'+CHAR(10)+'GO'+CHAR(10),'%LOGIN%',SYSTEM_USER)
    PRINT @lcLIST
    END
 ELSE BEGIN
    PRINT REPLACE('USER %LOGIN% has "ALTER TRACE permissions"', '%LOGIN%',SYSTEM_USER)
 END

 IF (SELECT SUM(CASE WHEN VALUE=1 THEN 1 ELSE 0 END) FROM sys.configurations WHERe NAME IN ('show advanced options','Ole Automation Procedures'))<2 BEGIN
    SET @lcLIST='/*****************************************/'+CHAR(10)+
                '/** RUN this                            **/'+CHAR(10)+
                '/*****************************************/'+CHAR(10)+
                'sp_configure ''show advanced options'', 1;'+CHAR(10)+
                'GO'+CHAR(10)+
                'RECONFIGURE; '+CHAR(10)+
                'GO'+CHAR(10)+
                'sp_configure ''Ole Automation Procedures'', 1;'+CHAR(10)+
                'GO'+CHAR(10)+
                'RECONFIGURE; '+CHAR(10)+
                'GO'+CHAR(10)

    PRINT @lcLIST
    END
 ELSE BEGIN
    PRINT 'Server has "Ole Automation Procedures" set to 1'
 END

 SET @lcLIST=''
 SET @lorsData=CURSOR LOCAL READ_ONLY FOR
  SELECT BA.name, ISNULL(BB.ISEXISTS,0)
    FROM (SELECT 'sp_OACreate' AS NAME
          UNION 
          SELECT 'sp_OADestroy'
          UNION 
          SELECT 'sp_OAGetErrorInfo'
          UNION 
          SELECT 'sp_OAMethod') BA LEFT JOIN (SELECT AB.name, CASE WHEN AA.permission_name='EXECUTE' THEN 1 ELSE 0 END AS ISEXISTS
                                                FROM master.sys.database_permissions AA JOIN sys.system_objects AB ON AA.major_id = AB.object_id
                                                WHERE AA.class = 1 AND AB.parent_object_id = 0 AND AB.name IN ('sp_OACreate','sp_OADestroy','sp_OAGetErrorInfo','sp_OAMethod')
                                              ) BB ON BA.NAME=BB.NAME
 OPEN @lorsData
 WHILE 0 = 0 BEGIN
   FETCH NEXT FROM @lorsData INTO @lcNAME, @liEXISTS
   IF @@FETCH_STATUS <> 0 BREAK

   PRINT @lcNAME+' '+CASE WHEN @liEXISTS=1 THEN 'GRANTED' ELSE 'NONGRANTED' END
 
   IF @liEXISTS=0 BEGIN
      SET @lcLIST=@lcLIST+'GRANT EXEC ON '+@lcNAME+' TO ['+SYSTEM_USER+']'+CHAR(10)+'GO'+CHAR(10)
   END
   
 END
 CLOSE @lorsData
 DEALLOCATE @lorsData

 IF LEN(@lcLIST)>0 BEGIN
    PRINT ''
    PRINT 'Missing grant of procedure'
    PRINT '/*****************************************/'+CHAR(10)+
          '/** RUN this                            **/'+CHAR(10)+
          '/*****************************************/'+CHAR(10)+
          'use master'+CHAR(10)+
		  @lcLIST
    PRINT ''
 END



END --_CVP_CHECKGRANT
GO

--exec dbo._CVP_CHECKGRANT


IF NOT OBJECT_ID('dbo._CVP_SAVESRCS') IS NULL DROP PROCEDURE dbo._CVP_SAVESRCS
GO
CREATE PROCEDURE dbo._CVP_SAVESRCS(@lcFOLDER NVARCHAR(1000), @lcSHORTFILENAME NVARCHAR(1000), @liMODE INT=1)
AS
BEGIN
 DECLARE @lorsData CURSOR, @lorsDB CURSOR
 DECLARE @liErr INT, @liErrX INT, @loFSO INT, @loTXT INT, @lcMSG VARCHAR(MAX), @liFLAG INT, @lcFile VARCHAR(2000),
         @lcLINE VARCHAR(4000), @liS BIGINT, @liDATABASEID INT, @lcNAME NVARCHAR(256), @liID INT, 
         @loLOGIF INT, @loLOGIP INT, @loLOGIS INT, @lcLOGIF VARCHAR(2000), @lcLOGIP VARCHAR(2000), @lcLOGIS VARCHAR(2000),
         @liDBID INT, @liPID INT,@liOBJECTID INT

 DECLARE @lcFILENAME VARCHAR(100), @lcTEXT VARCHAR(MAX), @liTLEN BIGINT

BEGIN TRY

 EXECUTE @liErr = sp_OACreate  'Scripting.FileSystemObject' , @loFSO OUT

 IF @liErr<>0 GOTO _CVP_SAVESRCS_END

 SET @lcFOLDER=@lcFOLDER+CASE WHEN RIGHT(@lcFOLDER,1)<>'\' THEN '\' ELSE '' END


 -- kontrola zda existuje sloka
 EXECUTE @liErr = sp_OAMethod   @loFSO, 'FolderExists',  @liFLAG OUT, @lcFOLDER
 IF @liErr<>0 BEGIN 
    GOTO _CVP_SAVESRCS_END
 END 
 IF @liFLAG=0 BEGIN
    SET @lcMSG=REPLACE('Neexistuje adres pro uloen soubor %DIR%. Export zruen.','%DIR%',@lcFOLDER)
    PRINT @lcMSG
    GOTO _CVP_SAVESRCS_END
 END

 -- vytvoen statistickho souboru
 SET @lcLOGIS=@lcFOLDER+@lcSHORTFILENAME+'.logis'
 EXECUTE @liErr = sp_OAMethod   @loFSO, 'CreateTextFile',  @loLOGIS OUT, @lcLOGIS, 2, 0
 IF @liErr<>0 BEGIN 
    SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%.','%FILE%',@lcLOGIS)
    PRINT @lcMSG
    GOTO _CVP_SAVESRCS_TXT
 END 
 SET @lcLINE=(SELECT CAST(DATABASEIDLEN AS VARCHAR(30))+','+CAST(OBJECTIDLEN AS VARCHAR(30))+','+CAST(NESTLEVELLEN AS VARCHAR(30)) FROM #STATS)
 EXECUTE @liErr = sp_OAMethod  @loLOGIS, 'WriteLine', Null, @lcLINE
 IF @liErr=0 BEGIN
    SET @lcMSG=REPLACE('Export dat do souboru %FILE%.','%FILE%',@lcLOGIS)
    PRINT @lcMSG
 END
 EXECUTE @liErrX = sp_OAMethod  @loLOGIS, 'Close'
 IF @liErr=0 BEGIN
    EXECUTE @liErr = sp_OAMethod   @loFSO, 'DELETEFILE',  NULL, @lcFOLDER, 1
 END
 EXECUTE sp_OADestroy @loTXT

 _CVP_SAVESRCS_TXT:

 IF @liMODE=2 BEGIN
    SET @lcLOGIF=@lcFOLDER+@lcSHORTFILENAME+'.logif'
    SET @lcLOGIP=@lcFOLDER+@lcSHORTFILENAME+'.logip'

    -- vytvoen textovho souboru
    EXECUTE @liErr = sp_OAMethod   @loFSO, 'CreateTextFile',  @loLOGIF OUT, @lcLOGIF, 2, 0
    IF @liErr<>0 BEGIN 
       SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%. Export zruen.','%FILE%',@lcLOGIF)
       PRINT @lcMSG
       GOTO _CVP_SAVESRCS_END
    END 

    -- vytvoen textovho souboru
    EXECUTE @liErr = sp_OAMethod   @loFSO, 'CreateTextFile',  @loLOGIP OUT, @lcLOGIP, 2, 0
    IF @liErr<>0 BEGIN 
       SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%. Export zruen.','%FILE%',@lcLOGIP)
       PRINT @lcMSG
       GOTO _CVP_SAVESRCS_END
    END 
 END

 SET @lorsDB=CURSOR LOCAL READ_ONLY FOR
   SELECT ID, DATABASEID, FILENAME
     FROM #DBs
     WHERE DATABASEID IN (SELECT DATABASEID FROM #OBJECTs)
 

 OPEN @lorsDB
 WHILE 0 = 0 BEGIN
   FETCH NEXT FROM @lorsDB INTO @liDBID, @liDATABASEID, @lcFILENAME
   IF @@FETCH_STATUS <> 0 BREAK

   SET @lcFile=@lcFOLDER+@lcFILENAME

   IF @liMODE=2 BEGIN
      SET @lcLINE=CAST(@liDBID AS VARCHAR(30))+','+@lcFILENAME+','+CAST(@liDATABASEID AS VARCHAR(30))
      EXECUTE @liErr = sp_OAMethod  @loLOGIF, 'WriteLine', Null, @lcLINE
      IF @liErr<>0 BEGIN PRINT 'FUCK' BREAK END
   END

   -- vytvoen textovho souboru
   EXECUTE @liErr = sp_OAMethod   @loFSO, 'CreateTextFile',  @loTXT OUT, @lcFile, 2, 0
   IF @liErr<>0 BEGIN 
      SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%. Export zruen.','%FILE%',@lcFile)
      PRINT @lcMSG
      BREAK
   END 

   SET @lorsData=CURSOR LOCAL READ_ONLY FOR
     SELECT ID, SOURCE, LEN('a'+SOURCE+'a')-2 AS TLEN, OWNER+'.'+NAME, OBJECTID
       FROM #OBJECTs 
       WHERE DATABASEID=@liDATABASEID
       ORDER BY OWNER,NAME

  OPEN @lorsData
   WHILE 0 = 0 BEGIN
     FETCH NEXT FROM @lorsData INTO @liPID, @lcTEXT, @liTLEN, @lcNAME, @liOBJECTID
     IF @@FETCH_STATUS <> 0 BREAK

     IF @liMODE=2 BEGIN
        SET @lcLINE=CAST(@liDBID AS VARCHAR(30))+','+CAST(@liPID AS VARCHAR(30))+',,'+@lcNAME+','+CAST(@liOBJECTID AS VARCHAR(30))
        EXECUTE @liErr = sp_OAMethod  @loLOGIP, 'WriteLine', Null, @lcLINE
        IF @liErr<>0 BEGIN PRINT 'FUCK' BREAK END
     END

     IF @liTLEN<=4000 AND RIGHT(@lcTEXT,1)<>CHAR(10) BEGIN
        EXECUTE @liErr = sp_OAMethod  @loTXT, 'WriteLine', Null, @lcTEXT
        IF @liErr<>0 BEGIN PRINT 'FUCK' BREAK END
     END
     IF @liTLEN<=4000 AND RIGHT(@lcTEXT,1)=CHAR(10) BEGIN
        EXECUTE @liErr = sp_OAMethod  @loTXT, 'Write', Null, @lcTEXT
        IF @liErr<>0 BEGIN PRINT 'FUCK' BREAK END
     END
     IF @liTLEN>4000  BEGIN
        SET @liS=0
        WHILE @liS<=@liTLEN BEGIN
          SET @lcLINE=SUBSTRING(@lcTEXT,@liS+1,4000)

          EXECUTE @liErr = sp_OAMethod  @loTXT, 'Write', Null, @lcLINE
          IF @liErr<>0 BEGIN PRINT 'FUCK' BREAK END

          SET @liS=@liS+4000
        END 
     END
     --END OF @loIFT061_TLEN>4000
     EXECUTE @liErr = sp_OAMethod  @loTXT, 'WriteLine', Null, ''
     IF @liErr<>0 BEGIN PRINT 'FUCK' BREAK END

   END
   CLOSE @lorsData
   DEALLOCATE @lorsData



   -- zavi vstupn soubor
   EXECUTE @liErrX = sp_OAMethod  @loTXT, 'Close'
   SET @liErr=@liErr|@liErrX
   EXECUTE sp_OADestroy @loTXT

   IF @liErr=0 BEGIN
      SET @lcMSG=REPLACE('Export dat do souboru %FILE%.','%FILE%',@lcFile)
      PRINT @lcMSG
   END

   IF @liErr<>0 BEGIN
      SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%. Export zruen.','%FILE%',@lcFile)
      PRINT @lcMSG
   END


 END
 CLOSE @lorsDB
 DEALLOCATE @lorsDB


 IF @liMODE=2 BEGIN
    EXECUTE @liErr = sp_OAMethod  @loLOGIF, 'Close'
    EXECUTE sp_OADestroy @loLOGIF

    IF @liErr=0 BEGIN
       SET @lcMSG=REPLACE('Export dat do souboru %FILE%.','%FILE%',@lcLOGIF)
       PRINT @lcMSG
    END
    IF @liErr<>0 BEGIN
       SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%. Export zruen.','%FILE%',@lcLOGIF)
       PRINT @lcMSG
    END

    EXECUTE @liErr = sp_OAMethod  @loLOGIP, 'Close'
    EXECUTE sp_OADestroy @loLOGIP

    IF @liErr=0 BEGIN
       SET @lcMSG=REPLACE('Export dat do souboru %FILE%.','%FILE%',@lcLOGIP)
       PRINT @lcMSG
    END
    IF @liErr<>0 BEGIN
       SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%. Export zruen.','%FILE%',@lcLOGIP)
       PRINT @lcMSG
    END

 END




 _CVP_SAVESRCS_END:
 EXECUTE sp_OADestroy @loFSO

END TRY

BEGIN CATCH
 SET @lcMSG=CAST(ERROR_PROCEDURE() AS VARCHAR(30))+' '+CAST(ERROR_NUMBER() AS VARCHAR(30))+' '+ERROR_MESSAGE()
 PRINT '***********************************************************'
 PRINT @lcMSG
 PRINT '***********************************************************'
END CATCH

END --_CVP_SAVESRCS
GO

IF NOT OBJECT_ID('dbo._CVP_SAVELOG') IS NULL DROP PROCEDURE dbo._CVP_SAVELOG
GO
CREATE PROCEDURE dbo._CVP_SAVELOG(@lcFOLDER NVARCHAR(1000), @lcSHORTFILENAME NVARCHAR(1000), @lcTABLENAME NVARCHAR(128), @liMODE INT=1, @liMAXNESTLEVEL INT=0 OUT)
AS
BEGIN
 DECLARE @lorsData CURSOR
 DECLARE @liErr INT, @liErrX INT, @loFSO INT, @loTXT INT, @lcMSG VARCHAR(MAX), @liFLAG INT, @lcFile VARCHAR(2000),
         @lcLINE VARCHAR(4000), @liS BIGINT, @lcBUFFER VARCHAR(4000), @liNESTLEVELZ INT

 DECLARE @lcTEXT VARCHAR(MAX), @liTLEN BIGINT, @lcSQL NVARCHAR(MAX)
 DECLARE @ldSTARTTIME DATETIME, @lcPROC NVARCHAR(256), @liLINENUMBER INT, @lcFILENAME NVARCHAR(1000), 
         @liLVL INT, @lnDURATION NUMERIC(20,6), @liROWNUMBER INT, @liREQUESTID INT


 EXECUTE @liErr = sp_OACreate  'Scripting.FileSystemObject' , @loFSO OUT

 IF @liErr<>0 GOTO _CVP_SAVELOG_END

 SET @lcFOLDER=@lcFOLDER+CASE WHEN RIGHT(@lcFOLDER,1)<>'\' THEN '\' ELSE '' END

 SET @lcFile=@lcFOLDER+@lcSHORTFILENAME+'.log'

 -- kontrola zda existuje sloka
 EXECUTE @liErr = sp_OAMethod   @loFSO, 'FolderExists',  @liFLAG OUT, @lcFOLDER
 IF @liErr<>0 BEGIN 
    GOTO _CVP_SAVELOG_END
 END 
 IF @liFLAG=0 BEGIN
    SET @lcMSG=REPLACE('Neexistuje adres pro uloen soubor %DIR%. Export zruen.','%DIR%',@lcFOLDER)
    PRINT @lcMSG
    GOTO _CVP_SAVELOG_END
 END

 -- vytvoen textovho souboru
 EXECUTE @liErr = sp_OAMethod   @loFSO, 'CreateTextFile',  @loTXT OUT, @lcFile, 2, 0
 IF @liErr<>0 BEGIN 
    SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%. Export zruen.','%FILE%',@lcFile)
    PRINT @lcMSG
    GOTO _CVP_SAVELOG_END
 END 

 --SET @lcSQL='
 --     SELECT AG.DURATION/1000000.0000, AB.OWNER+''.''+AB.NAME, AE.LINENUMBER, AF.FILENAME, AA.NESTLEVEL, AA.REQUESTID
 --       FROM '+@lcTABLENAME+' AA LEFT JOIN #OBJECTs  AB ON AA.DATABASEID=AB.DATABASEID AND AA.OBJECTID=AB.OBJECTID
 --                          LEFT JOIN #OBJECTSRCs AE ON AA.DATABASEID=AE.DATABASEID AND AA.OBJECTID=AE.OBJECTID AND AA.LINENUMBER=AE.ORD
 --                          LEFT JOIN #DBs        AF ON AA.DATABASEID=AF.DATABASEID
 --                          LEFT JOIN '+@lcTABLENAME+' AG ON AG.EVENTSEQUENCE=(SELECT MIN(BB.EVENTSEQUENCE)
 --                                                                                FROM '+@lcTABLENAME+' BB
 --                                                                                WHERE AA.OFFSET=BB.OFFSET AND BB.EVENTSEQUENCE>AA.EVENTSEQUENCE AND EVENTCLASS=45
 --                                                                              ) AND AG.EVENTCLASS=45
 --       WHERE AA.EVENTCLASS=44
 --       ORDER BY AA.EVENTSEQUENCE
 --    '
 --EXEC (@lcSQL)



 -- Vytvo dle zdrojov tabulky log soubor pro VFP
 IF @liMODE=1 BEGIN
    SET @lcSQL='SET @loCursor = CURSOR LOCAL READ_ONLY FOR 
      SELECT AG.DURATION/1000000.0000, AB.OWNER+''.''+AB.NAME, AE.LINENUMBER, AF.FILENAME, AA.NESTLEVEL, AA.REQUESTID
        FROM '+@lcTABLENAME+' AA INNER JOIN #OBJECTs  AB ON AA.DATABASEID=AB.DATABASEID AND AA.OBJECTID=AB.OBJECTID
                           INNER JOIN #OBJECTSRCs AE ON AA.DATABASEID=AE.DATABASEID AND AA.OBJECTID=AE.OBJECTID AND AA.LINENUMBER=AE.ORD
                           INNER JOIN #DBs        AF ON AA.DATABASEID=AF.DATABASEID
                           INNER JOIN '+@lcTABLENAME+' AG ON AG.EVENTSEQUENCE=(SELECT MIN(BB.EVENTSEQUENCE)
                                                                                 FROM '+@lcTABLENAME+' BB
                                                                                 WHERE AA.OFFSET=BB.OFFSET AND BB.EVENTSEQUENCE>AA.EVENTSEQUENCE AND EVENTCLASS=45
                                                                               ) AND AG.EVENTCLASS=45
        WHERE AA.EVENTCLASS=44
        ORDER BY AA.EVENTSEQUENCE
        ;
       OPEN @loCursor'
 END

 IF @liMODE=2 BEGIN
    SET @lcSQL='SET @loCursor = CURSOR LOCAL READ_ONLY FOR 
      SELECT AG.DURATION/1000000.0000, AB.ID, AE.LINENUMBER, AF.ID, AA.NESTLEVEL, AA.REQUESTID
        FROM '+@lcTABLENAME+' AA INNER JOIN #OBJECTs  AB ON AA.DATABASEID=AB.DATABASEID AND AA.OBJECTID=AB.OBJECTID
                           INNER JOIN #OBJECTSRCs AE ON AA.DATABASEID=AE.DATABASEID AND AA.OBJECTID=AE.OBJECTID AND AA.LINENUMBER=AE.ORD
                           INNER JOIN #DBs        AF ON AA.DATABASEID=AF.DATABASEID
                           INNER JOIN '+@lcTABLENAME+' AG ON AG.EVENTSEQUENCE=(SELECT MIN(BB.EVENTSEQUENCE)
                                                                                 FROM '+@lcTABLENAME+' BB
                                                                                 WHERE AA.OFFSET=BB.OFFSET AND BB.EVENTSEQUENCE>AA.EVENTSEQUENCE AND EVENTCLASS=45
                                                                               ) AND AG.EVENTCLASS=45
        WHERE AA.EVENTCLASS=44
        ORDER BY AA.EVENTSEQUENCE
        ;
       OPEN @loCursor'
 END

 EXEC sp_executesql @lcSQL, N'@loCursor CURSOR OUTPUT', @lorsData OUTPUT

 SET @liROWNUMBER=0
 SET @lcBUFFER=''
 SET @liNESTLEVELZ=0
 WHILE 0 = 0 BEGIN
   FETCH NEXT FROM @lorsData INTO @lnDURATION, @lcPROC, @liLINENUMBER, @lcFILENAME, @liLVL, @liREQUESTID
   IF @@FETCH_STATUS <> 0 BREAK


      SELECT @liMAXNESTLEVEL= CASE WHEN @liMAXNESTLEVEL<LVL THEN LVL ELSE @liMAXNESTLEVEL END,
             @liNESTLEVELZ=NESTLEVELZ, @liLVL=LVL
        FROM (SELECT CASE WHEN @liREQUESTID=0 THEN @liLVL ELSE @liNESTLEVELZ END AS NESTLEVELZ,
                     CASE WHEN @liREQUESTID<>0 THEN @liLVL+@liNESTLEVELZ ELSE @liLVL END AS LVL
             ) AA



   --IF @liREQUESTID=0 BEGIN
   --   SET @liNESTLEVELZ=@liLVL
   --END
   --IF @liREQUESTID<>0 BEGIN
   --   SET @liLVL=@liLVL+@liNESTLEVELZ
   --END

   --IF @liMAXNESTLEVEL<@liLVL BEGIN
   --   SET @liMAXNESTLEVEL=@liLVL
   --END

   SET @liROWNUMBER=@liROWNUMBER+1
   SET @lcLINE=CONVERT(VARCHAR(20),@lnDURATION,0)+',,'+@lcPROC+','+CONVERT(VARCHAR(30),@liLINENUMBER,0)+','+@lcFILENAME+','+CONVERT(VARCHAR(30),@liLVL,0)+CHAR(13)+CHAR(10)

   IF LEN(@lcBUFFER)+LEN(@lcLINE)>4000 BEGIN
      EXECUTE @liErr = sp_OAMethod  @loTXT, 'Write', Null, @lcBUFFER
      IF @liErr<>0 BEGIN PRINT 'FUCK' BREAK END
      SET @lcBUFFER=''
      PRINT @liROWNUMBER
   END
   SET @lcBUFFER=@lcBUFFER+@lcLINE
 END
 CLOSE @lorsData
 DEALLOCATE @lorsData

 IF LEN(@lcBUFFER)>0 BEGIN
    EXECUTE @liErr = sp_OAMethod  @loTXT, 'Write', Null, @lcBUFFER
    IF @liErr<>0 BEGIN PRINT 'FUCK' END
 END

 -- zavi vstupn soubor
 EXECUTE @liErrX = sp_OAMethod  @loTXT, 'Close'
 SET @liErr=@liErr|@liErrX
 EXECUTE sp_OADestroy @loTXT

 IF @liErr=0 BEGIN
    SET @lcMSG=REPLACE('Export dat do souboru %FILE%.','%FILE%',@lcFile)
    PRINT @lcMSG
 END

 IF @liErr<>0 BEGIN
    SET @lcMSG=REPLACE('Nelze vytvoit soubor %FILE%. Export zruen.','%FILE%',@lcFile)
    PRINT @lcMSG
 END


 _CVP_SAVELOG_END:
 EXECUTE sp_OADestroy @loFSO


END --_CVP_SAVELOG
GO

IF NOT OBJECT_ID('dbo._CVP_SS') IS NULL DROP FUNCTION dbo._CVP_SS
GO


IF NOT OBJECT_ID('dbo._CVP_PROCESS') IS NULL DROP PROCEDURE dbo._CVP_PROCESS
GO
CREATE PROCEDURE dbo._CVP_PROCESS(@lcTABLENAME NVARCHAR(128), @lcFOLDER NVARCHAR(1000), @lcSHORTFILENAME NVARCHAR(1000), @liMODE INT=1)
AS
BEGIN

 DECLARE @lorsDB CURSOR, @lorsOBJECT CURSOR
 DECLARE @lcSQL NVARCHAR(MAX), @lcSOURCE NVARCHAR(MAX)
 DECLARE @liDATABASEID INT, @lcNAME NVARCHAR(128), @liOBJECTID INT, @liCNT INT, @liROWS INT, @liMAXNESTLEVEL INT

 CREATE TABLE #DBs (ID INT, DATABASEID INT, NAME NVARCHAR(128), FILENAME NVARCHAR(100))
 CREATE TABLE #OBJECTs (ID INT, DATABASEID INT, OBJECTID INT, OWNER NVARCHAR(128), NAME NVARCHAR(128), SOURCE NVARCHAR(MAX), UNIQUE CLUSTERED (DATABASEID, OBJECTID))
 CREATE TABLE #OBJECTSRCs (DATABASEID INT, OBJECTID INT, LINENUMBER INT, ORD INT, UNIQUE CLUSTERED (DATABASEID, OBJECTID, ORD))
 CREATE TABLE #STATS (DATABASEIDLEN INT, OBJECTIDLEN INT, NESTLEVELLEN INT)

 -- zjitn databz
 SET @lcSQL=
 'INSERT INTO #DBs '+
 '  SELECT ROW_NUMBER() OVER( ORDER BY AB.NAME), AA.DATABASEID, AB.NAME, REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(REPLACE('''+@lcSHORTFILENAME+'_''+CAST(AA.DATABASEID AS VARCHAR(30)),''['',''''), ''\'',''''), ''/'',''''), '':'',''''), ''*'',''''), ''?'',''''), ''"'',''''), ''<'',''''), ''>'',''''), ''|'',''''), '']'','''')+''.tsql'''+
 '    FROM (SELECT DISTINCT DATABASEID FROM '+@lcTABLENAME+' WHERE EVENTCLASS=44'+
 '         ) AA INNER JOIN sys.databases  AB ON AA.databaseid=AB.database_id'
 EXECUTE (@lcSQL)



 -- zjitn procedur a funkc v databzch
 SET @lorsDB=CURSOR READ_ONLY STATIC FOR
   SELECT DATABASEID, NAME FROM #DBs
 OPEN @lorsDB
 WHILE 0 = 0 BEGIN
   FETCH NEXT FROM @lorsDB INTO @liDATABASEID, @lcNAME
   IF @@FETCH_STATUS <> 0 BREAK

   SET @lcSQL=
   'INSERT INTO #OBJECTs '+
   '  SELECT ROW_NUMBER() OVER( ORDER BY AC.NAME, AB.NAME), AA.DATABASEID, AA.OBJECTID, AC.NAME, AB.NAME, AD.[definition]'+
   '    FROM (SELECT DISTINCT DATABASEID, OBJECTID FROM '+@lcTABLENAME+' WHERE DATABASEID='+CAST(@liDATABASEID AS VARCHAR(30))+' AND EVENTCLASS=44'+
   '         ) AA INNER JOIN '+@lcNAME+'.sys.objects         AB ON AA.objectid=AB.object_id'+
   '              INNER JOIN '+@lcNAME+'.sys.schemas         AC ON AB.schema_id=AC.schema_id'+
   '              INNER JOIN '+@lcNAME+'.sys.all_sql_modules AD ON AA.objectid=AD.object_id'+
   '    ORDER BY AC.NAME, AB.NAME'
   EXECUTE (@lcSQL)

 END 
 CLOSE @lorsDB
 DEALLOCATE @lorsDB


 SET @liCNT=0
 -- zjitn zdrojovch kd procedur a funkc v databzch
 SET @lorsOBJECT=CURSOR READ_ONLY STATIC FOR
   SELECT DATABASEID, OBJECTID, LEN(SOURCE)-LEN(REPLACE(SOURCE,CHAR(10),''))+1
     FROM #OBJECTs
     ORDER BY OWNER, NAME
 OPEN @lorsOBJECT
 WHILE 0 = 0 BEGIN
   FETCH NEXT FROM @lorsOBJECT INTO @liDATABASEID, @liOBJECTID, @liROWS
   IF @@FETCH_STATUS <> 0 BREAK;

  WITH tree(id) AS
   (SELECT 1
    UNION ALL
    SELECT AA.ID+AB.ID 
      FROM (SELECT 1 AS ID) AA INNER JOIN TREE AB ON 0=0
      WHERE AA.ID+AB.ID<=@LIROWS
   )
   INSERT INTO #OBJECTSRCs
     SELECT @liDATABASEID, @liOBJECTID, @liCNT+ID, ID FROM tree
     OPTION (MAXRECURSION 32000)
   SET @liCNT=@liCNT+@@ROWCOUNT

 END 
 CLOSE @lorsOBJECT
 DEALLOCATE @lorsOBJECT


 -- Vytvo log dle zdrojovho souboru
 SET @liMAXNESTLEVEL=0
 EXEC dbo._CVP_SAVELOG @lcFOLDER, @lcSHORTFILENAME, @lcTABLENAME, @liMODE, @liMAXNESTLEVEL OUT

 INSERT INTO #STATS (DATABASEIDLEN, OBJECTIDLEN, NESTLEVELLEN)
  SELECT MAX(LEN(CAST(AB.ID AS VARCHAR(30)))), MAX(LEN(CAST(AA.ID AS VARCHAR(30)))), MAX(LEN(CAST(@liMAXNESTLEVEL AS VARCHAR(30))))
   FROM #OBJECTs AA INNER JOIN #DBs AB ON AA.DATABASEID=AB.DATABASEID

 
 -- Vytvo soubory dle natench OBJECTs a napl je zdrojovma kdama
 EXEC dbo._CVP_SAVESRCS @lcFOLDER, @lcSHORTFILENAME, @liMODE



 IF NOT OBJECT_ID('tempdb.dbo.#DBs') IS NULL DROP TABLE #DBs
 IF NOT OBJECT_ID('tempdb.dbo.#OBJECTs') IS NULL DROP TABLE #OBJECTs
 IF NOT OBJECT_ID('tempdb.dbo.#OBJECTSRCs') IS NULL DROP TABLE #OBJECTSRCs
 IF NOT OBJECT_ID('tempdb.dbo.#STATS') IS NULL DROP TABLE #STATS
END --_CVP_Process
GO



IF NOT OBJECT_ID('dbo._CVP_PROCESS_TRC') IS NULL DROP PROCEDURE dbo._CVP_PROCESS_TRC
GO
CREATE PROCEDURE dbo._CVP_PROCESS_TRC(@lcTRC NVARCHAR(1000), @lcFOLDER NVARCHAR(1000), @lcSHORTFILENAME NVARCHAR(1000), @liMODE INT=1)
AS
BEGIN
 DECLARE @ltT1 DATETIME

 SET @ltT1=GETDATE()
 SELECT  eventclass, Databaseid, Linenumber, Duration, Objectid , NESTLEVEL, REQUESTID, EVENTSEQUENCE, OFFSET, SPID
   INTO #TRC
   FROM ::fn_trace_gettable(@lcTRC, DEFAULT)
   WHERE STATE IS NULL AND eventclass IN (44,45)
 PRINT 'Load '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))

 SET @ltT1=GETDATE()
 CREATE INDEX #IN1_TRC ON #TRC (DATABASEID, OBJECTID) WHERE EVENTCLASS=44
 PRINT '#IN1 '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))

 SET @ltT1=GETDATE()
 CREATE INDEX #IN2_TRC ON #TRC (EVENTSEQUENCE) INCLUDE (DURATION) WHERE EVENTCLASS=45
 PRINT '#IN2 '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))

 SET @ltT1=GETDATE()
 CREATE INDEX #IN3_TRC ON #TRC (OFFSET, EVENTSEQUENCE)  WHERE EVENTCLASS=45
 PRINT '#IN3 '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))

 SET @ltT1=GETDATE()
 EXEC _CVP_Process '#TRC', @lcFOLDER,  @lcSHORTFILENAME, @liMODE
 PRINT '_CVP_Process '+CAST(DATEDIFF(ms,@ltT1,GETDATE())/1000.0 AS VARCHAR(30))

 --SELECT TOP 1000 * FROM #TRC ORDER BY EVENTSEQUENCE


 IF NOT OBJECT_ID('tempdb.dbo.#TRC') IS NULL DROP TABLE #TRC

END --_CVP_Process_TRC
GO


/*
EXEC _CVP_Process_TRC '\\egeterminal3\Vyroba\PRODUKTY\EGSYS\Nstroje\MSSQL CVP\TRC\I2_TV_A_102.trc', '\\egeterminal3\Vyroba\PRODUKTY\EGSYS\Nstroje\MSSQL CVP\LOG', 'I2_TV_A_102',1
EXEC _CVP_Process_TRC '\\EXPERT54\MSPROFILER\TRC\_CVP_000.trc', '\\EXPERT54\MSPROFILER\log', '_CVP_000',2

EXEC _CVP_Process_TRC '\\EXPERT54\MSPROFILER\TRC\I2_SILKA_A_XXX.trc', '\\EXPERT54\MSPROFILER\log', 'I2_SILKA_A_XXX',2

EXEC _CVP_Process_TRC '\\EXPERT54\MSPROFILER\TRC\test5.trc', '\\EXPERT54\MSPROFILER\log', 'test5',2


exec dbo._CVP_CHECKGRANT

*/